@@ -9,7 +9,7 @@
 #include <OSL/oslconfig.h>
 
 
-OSL_NAMESPACE_ENTER
+OSL_NAMESPACE_BEGIN
 namespace pvt {
 
 class ASTNode;
@@ -98,6 +98,10 @@ public:
     /// is not responsible for freeing the characters.
     const char* c_str() const;
 
+    /// Return the c_str giving a human-readable name of a type, fully
+    /// accounting for exotic types like structs, etc.
+    const char* type_c_str() const;
+
     /// Stream output
     friend std::ostream& operator<<(std::ostream& o, const TypeSpec& t)
     {
@@ -244,46 +248,25 @@ public:
     // m_simple is always UNKNOWN for structures.
 
     /// Is it a simple scalar int?
-    bool is_int() const
-    {
-        return m_simple == TypeDesc::TypeInt && !is_closure();
-    }
+    bool is_int() const { return m_simple == TypeInt && !is_closure(); }
 
     /// Is it a simple scalar float?
-    bool is_float() const
-    {
-        return m_simple == TypeDesc::TypeFloat && !is_closure();
-    }
+    bool is_float() const { return m_simple == TypeFloat && !is_closure(); }
 
     /// Is it a color?
-    bool is_color() const
-    {
-        return m_simple == TypeDesc::TypeColor && !is_closure();
-    }
+    bool is_color() const { return m_simple == TypeColor && !is_closure(); }
 
     /// Is it a point?
-    bool is_point() const
-    {
-        return m_simple == TypeDesc::TypePoint && !is_closure();
-    }
+    bool is_point() const { return m_simple == TypePoint && !is_closure(); }
 
     /// Is it a vector?
-    bool is_vector() const
-    {
-        return m_simple == TypeDesc::TypeVector && !is_closure();
-    }
+    bool is_vector() const { return m_simple == TypeVector && !is_closure(); }
 
     /// Is it a normal?
-    bool is_normal() const
-    {
-        return m_simple == TypeDesc::TypeNormal && !is_closure();
-    }
+    bool is_normal() const { return m_simple == TypeNormal && !is_closure(); }
 
     /// Is it a simple string?
-    bool is_string() const
-    {
-        return m_simple == TypeDesc::TypeString && !is_closure();
-    }
+    bool is_string() const { return m_simple == TypeString && !is_closure(); }
 
     /// Is it a string or an array of strings?
     ///
@@ -357,9 +340,8 @@ public:
     bool is_vectriple() const
     {
         return !is_closure()
-               && (m_simple == TypeDesc::TypePoint
-                   || m_simple == TypeDesc::TypeVector
-                   || m_simple == TypeDesc::TypeNormal);
+               && (m_simple == TypePoint || m_simple == TypeVector
+                   || m_simple == TypeNormal);
     }
 
     /// Is it based on a vector-like triple (point, vector, or normal)?
@@ -367,16 +349,12 @@ public:
     bool is_vectriple_based() const
     {
         auto elem = m_simple.elementtype();
-        return (elem == TypeDesc::TypePoint || elem == TypeDesc::TypeVector
-                || elem == TypeDesc::TypeNormal);
+        return (elem == TypePoint || elem == TypeVector || elem == TypeNormal);
     }
 
     /// Is it a simple matrix (but not an array or closure)?
     ///
-    bool is_matrix() const
-    {
-        return m_simple == TypeDesc::TypeMatrix && !is_closure();
-    }
+    bool is_matrix() const { return m_simple == TypeMatrix && !is_closure(); }
 
     /// Is it a color closure?
     ///
@@ -399,6 +377,28 @@ public:
                    && (src.is_float() || src.is_int()));
     }
 
+    /// Given a pointer to a type code string that we use for argument
+    /// checking ("p", "v", etc.) return the TypeSpec of the first type
+    /// described by the string (UNKNOWN if it couldn't be recognized).
+    /// If 'advance' is non-NULL, set *advance to the number of
+    /// characters taken by the first code so the caller can advance
+    /// their pointer to the next code in the string.
+    static TypeSpec type_from_code(const char* code, int* advance = nullptr);
+
+    /// Return the argument checking code ("p", "v", etc.) corresponding
+    /// to the type.
+    std::string code_from_type() const;
+
+    /// Take a type code string (possibly containing many types)
+    /// and turn it into a human-readable string.
+    static std::string typelist_from_code(const char* code);
+
+    /// Take a type code string (possibly containing many types) and
+    /// turn it into a TypeSpec vector.
+    static void typespecs_from_codes(const char* code,
+                                     std::vector<TypeSpec>& types);
+
+
 private:
     TypeDesc m_simple;  ///< Data if it's a simple type
     short m_structure;  ///< 0 is not a structure, >=1 for structure id
@@ -478,7 +478,9 @@ public:
         , m_const_initializer(false)
         , m_connected_down(false)
         , m_initialized(false)
-        , m_lockgeom(false)
+        , m_interpolated(false)
+        , m_interactive(false)
+        , m_noninteractive(false)
         , m_allowconnect(true)
         , m_renderer_output(false)
         , m_readonly(false)
@@ -630,137 +632,70 @@ public:
         OSL_ASSERT(arena == SymArena::Absolute);
         m_arena = static_cast<unsigned int>(arena);
         m_data  = ptr;
-        // m_dataoffset = static_cast<int64_t>((char*)ptr - (char*)0);
+    }
+
+    /// Specify the location of the symbol's data, relative to an arena
+    /// (which for now must be Absolute).
+    void set_dataptr(SymArena arena, void* ptr, int offset)
+    {
+        // OSL_ASSERT(arena == SymArena::Absolute);
+        m_arena      = static_cast<unsigned int>(arena);
+        m_data       = ptr;
+        m_dataoffset = offset;
+        OSL::print("setting sym {} arena {} offset {}\n", name(), int(m_arena),
+                   m_dataoffset);
     }
 
 
-    void dataoffset(int d)
-    {
-        m_dataoffset = d;
-    }
-    int dataoffset() const
-    {
-        return m_dataoffset;
-    }
+    void dataoffset(int d) { m_dataoffset = d; }
+    int dataoffset() const { return m_dataoffset; }
 
-    void wide_dataoffset(int d)
-    {
-        m_wide_dataoffset = d;
-    }
-    int wide_dataoffset() const
-    {
-        return m_wide_dataoffset;
-    }
+    void wide_dataoffset(int d) { m_wide_dataoffset = d; }
+    int wide_dataoffset() const { return m_wide_dataoffset; }
 
-    SymArena arena() const
-    {
-        return static_cast<SymArena>(m_arena);
-    }
+    SymArena arena() const { return static_cast<SymArena>(m_arena); }
 
-    void initializers(int d)
-    {
-        m_initializers = d;
-    }
-    int initializers() const
-    {
-        return m_initializers;
-    }
+    void initializers(int d) { m_initializers = d; }
+    int initializers() const { return m_initializers; }
 
-    bool has_derivs() const
-    {
-        return m_has_derivs;
-    }
-    void has_derivs(bool new_derivs)
-    {
-        m_has_derivs = new_derivs;
-    }
-    int size() const
-    {
-        return m_size;
-    }
-    void size(size_t newsize)
-    {
-        m_size = (int)newsize;
-    }
+    bool has_derivs() const { return m_has_derivs; }
+    void has_derivs(bool new_derivs) { m_has_derivs = new_derivs; }
+    int size() const { return m_size; }
+    void size(size_t newsize) { m_size = (int)newsize; }
 
     /// Return the size for each point, including derivs.
     ///
-    int derivsize() const
-    {
-        return m_has_derivs ? 3 * m_size : m_size;
-    }
+    int derivsize() const { return m_has_derivs ? 3 * m_size : m_size; }
 
-    bool connected() const
-    {
-        return valuesource() == ConnectedVal;
-    }
-    bool connected_down() const
-    {
-        return m_connected_down;
-    }
-    void connected_down(bool c)
-    {
-        m_connected_down = c;
-    }
+    bool connected() const { return valuesource() == ConnectedVal; }
+    bool connected_down() const { return m_connected_down; }
+    void connected_down(bool c) { m_connected_down = c; }
 
     /// Where did the symbol's value come from?
     ///
     enum ValueSource { DefaultVal, InstanceVal, GeomVal, ConnectedVal };
 
-    ValueSource valuesource() const
-    {
-        return (ValueSource)m_valuesource;
-    }
-    void valuesource(ValueSource v)
-    {
-        m_valuesource = v;
-    }
+    ValueSource valuesource() const { return (ValueSource)m_valuesource; }
+    void valuesource(ValueSource v) { m_valuesource = v; }
     const char* valuesourcename() const;
     static const char* valuesourcename(ValueSource v);
 
-    int fieldid() const
-    {
-        return m_fieldid;
-    }
-    void fieldid(int id)
-    {
-        m_fieldid = id;
-    }
+    int fieldid() const { return m_fieldid; }
+    void fieldid(int id) { m_fieldid = id; }
 
-    int layer() const
-    {
-        return m_layer;
-    }
-    void layer(int id)
-    {
-        m_layer = id;
-    }
+    int layer() const { return m_layer; }
+    void layer(int id) { m_layer = id; }
 
-    int initbegin() const
-    {
-        return m_initbegin;
-    }
-    void initbegin(int i)
-    {
-        m_initbegin = i;
-    }
-    int initend() const
-    {
-        return m_initend;
-    }
-    void initend(int i)
-    {
-        m_initend = i;
-    }
+    int initbegin() const { return m_initbegin; }
+    void initbegin(int i) { m_initbegin = i; }
+    int initend() const { return m_initend; }
+    void initend(int i) { m_initend = i; }
     void set_initrange(int b = 0, int e = 0)
     {
         m_initbegin = b;
         m_initend   = e;
     }
-    bool has_init_ops() const
-    {
-        return m_initbegin != m_initend;
-    }
+    bool has_init_ops() const { return m_initbegin != m_initend; }
 
     /// Clear read/write usage info.
     ///
@@ -804,42 +739,15 @@ public:
         }
     }
 
-    int firstread() const
-    {
-        return m_firstread;
-    }
-    int lastread() const
-    {
-        return m_lastread;
-    }
-    int firstwrite() const
-    {
-        return m_firstwrite;
-    }
-    int lastwrite() const
-    {
-        return m_lastwrite;
-    }
-    int firstuse() const
-    {
-        return std::min(firstread(), firstwrite());
-    }
-    int lastuse() const
-    {
-        return std::max(lastread(), lastwrite());
-    }
-    bool everread() const
-    {
-        return lastread() >= 0;
-    }
-    bool everwritten() const
-    {
-        return lastwrite() >= 0;
-    }
-    bool everused() const
-    {
-        return everread() || everwritten();
-    }
+    int firstread() const { return m_firstread; }
+    int lastread() const { return m_lastread; }
+    int firstwrite() const { return m_firstwrite; }
+    int lastwrite() const { return m_lastwrite; }
+    int firstuse() const { return std::min(firstread(), firstwrite()); }
+    int lastuse() const { return std::max(lastread(), lastwrite()); }
+    bool everread() const { return lastread() >= 0; }
+    bool everwritten() const { return lastwrite() >= 0; }
+    bool everused() const { return everread() || everwritten(); }
     // everused_in_group is an even more stringent test -- not only must
     // the symbol not be used within the shader but it also must not be
     // used elsewhere in the group, by being connected to something downstream
@@ -860,67 +768,44 @@ public:
         m_lastwrite  = last;
     }
 
-    bool initialized() const
-    {
-        return m_initialized;
-    }
-    void initialized(bool init)
-    {
-        m_initialized = init;
-    }
+    bool initialized() const { return m_initialized; }
+    void initialized(bool init) { m_initialized = init; }
 
     bool lockgeom() const
     {
-        return m_lockgeom;
-    }
-    void lockgeom(bool lock)
-    {
-        m_lockgeom = lock;
+        // We can lock a value to a constant (at all places on all pieces of
+        // geometry) if it is neither interpolated nor interactively modified.
+        return !m_interpolated && !m_interactive;
     }
 
-    bool allowconnect() const
-    {
-        return m_allowconnect;
-    }
-    void allowconnect(bool val)
-    {
-        m_allowconnect = val;
-    }
+    bool interpolated() const { return m_interpolated; }
+    void interpolated(bool val) { m_interpolated = val; }
 
-    int arraylen() const
-    {
-        return m_typespec.arraylength();
-    }
+    bool interactive() const { return m_interactive; }
+    void interactive(bool val) { m_interactive = val; }
+
+    bool noninteractive() const { return m_noninteractive; }
+    void noninteractive(bool val) { m_noninteractive = val; }
+
+    bool allowconnect() const { return m_allowconnect; }
+    void allowconnect(bool val) { m_allowconnect = val; }
+
+    int arraylen() const { return m_typespec.arraylength(); }
     void arraylen(int len)
     {
         m_typespec.make_array(len);
         m_size = m_typespec.simpletype().size();
     }
 
-    bool renderer_output() const
-    {
-        return m_renderer_output;
-    }
-    void renderer_output(bool v)
-    {
-        m_renderer_output = v;
-    }
+    bool renderer_output() const { return m_renderer_output; }
+    void renderer_output(bool v) { m_renderer_output = v; }
 
     // When not uniform a symbol will have a varying value under batched
     // execution and must use a Wide data type to hold different values
     // for each data lane executing
-    bool is_uniform() const
-    {
-        return m_is_uniform;
-    }
-    bool is_varying() const
-    {
-        return (m_is_uniform == 0);
-    }
-    void make_varying()
-    {
-        m_is_uniform = false;
-    }
+    bool is_uniform() const { return m_is_uniform; }
+    bool is_varying() const { return (m_is_uniform == 0); }
+    void make_varying() { m_is_uniform = false; }
 
     // Results of a compare_op and other ops with logically boolean
     // results under certain conditions could be forced to be represented
@@ -934,32 +819,14 @@ public:
     // The value of forced_llvm_bool() is currently only respected during
     // batched execution.  Forced bools should not be coalesced with regular
     // ints, only other forced bools.
-    bool forced_llvm_bool() const
-    {
-        return m_forced_llvm_bool;
-    }
-    void forced_llvm_bool(bool v)
-    {
-        m_forced_llvm_bool = v;
-    }
+    bool forced_llvm_bool() const { return m_forced_llvm_bool; }
+    void forced_llvm_bool(bool v) { m_forced_llvm_bool = v; }
 
-    bool readonly() const
-    {
-        return m_readonly;
-    }
-    void readonly(bool v)
-    {
-        m_readonly = v;
-    }
+    bool readonly() const { return m_readonly; }
+    void readonly(bool v) { m_readonly = v; }
 
-    bool is_constant() const
-    {
-        return symtype() == SymTypeConst;
-    }
-    bool is_temp() const
-    {
-        return symtype() == SymTypeTemp;
-    }
+    bool is_constant() const { return symtype() == SymTypeConst; }
+    bool is_temp() const { return symtype() == SymTypeTemp; }
 
     // Retrieve the const float value (must be a const float!)
     float get_float(int index = 0) const
@@ -1029,10 +896,12 @@ protected:
     unsigned m_symtype : 4;     ///< Kind of symbol (param, local, etc.)
     unsigned m_has_derivs : 1;  ///< Step to derivs (0 == has no derivs)
     unsigned m_const_initializer : 1;  ///< initializer is a constant expression
-    unsigned m_connected_down : 1;   ///< Connected to a later/downstream layer
-    unsigned m_initialized : 1;      ///< If a param, has it been initialized?
-    unsigned m_lockgeom : 1;         ///< Is the param not overridden by geom?
-    unsigned m_allowconnect : 1;     ///< Is the param not overridden by geom?
+    unsigned m_connected_down : 1;  ///< Connected to a later/downstream layer
+    unsigned m_initialized : 1;     ///< If a param, has it been initialized?
+    unsigned m_interpolated : 1;    ///< Is the param overridden by geom?
+    unsigned m_interactive : 1;     ///< May the param change interactively?
+    unsigned m_noninteractive : 1;  ///< The param def won't modify interactively
+    unsigned m_allowconnect : 1;    ///< Is the param allowd to connect?
     unsigned m_renderer_output : 1;  ///< Is this sym a renderer output?
     unsigned m_readonly : 1;         ///< read-only symbol
     unsigned m_is_uniform : 1;  ///< symbol is uniform under batched execution
@@ -1282,7 +1151,26 @@ private:
 
 typedef std::vector<Opcode> OpcodeVec;
 
+/// Called after code is generated, this function loops over all the ops
+/// and figures out the lifetimes of all variables, based on whether the
+/// args in each op are read or written. This function is used both in
+/// the compiler and the runtime optimizer.
+void
+track_variable_lifetimes_main(const OpcodeVec& ircode,
+                              const SymbolPtrVec& opargs,
+                              const SymbolPtrVec& allsyms,
+                              std::vector<int>* bblock_ids = nullptr);
+
 
 
 };  // namespace pvt
-OSL_NAMESPACE_EXIT
+OSL_NAMESPACE_END
+
+
+
+// Supply a fmtlib compatible custom formatter for TypeSpec.
+#if FMT_VERSION >= 100000
+FMT_BEGIN_NAMESPACE
+template<> struct formatter<OSL::pvt::TypeSpec> : ostream_formatter {};
+FMT_END_NAMESPACE
+#endif
