@@ -16,7 +16,7 @@ using namespace OSL::pvt;
 #include <OSL/encodedtypes.h>
 #include <OSL/fmt_util.h>
 
-OSL_NAMESPACE_ENTER
+OSL_NAMESPACE_BEGIN
 
 
 #ifdef OIIO_TEXTURESYSTEM_CREATE_SHARED
@@ -176,6 +176,16 @@ RendererServices::get_array_attribute(ShaderGlobals* sg, bool derivatives,
 
 
 
+void
+RendererServices::build_interpolated_getter(const ShaderGroup& group,
+                                            const ustring& param_name,
+                                            TypeDesc type, bool derivatives,
+                                            InterpolatedGetterSpec& spec)
+{
+}
+
+
+
 bool
 RendererServices::get_userdata(bool derivatives, ustringhash name,
                                TypeDesc type, ShaderGlobals* sg, void* val)
@@ -436,6 +446,13 @@ RendererServices::get_texture_info(ustringhash filename,
                                                  texture_thread_info, subimage,
                                                  ustring_from(dataname),
                                                  datatype, data);
+    if (status && (datatype == TypeDesc::STRING)) {
+        // The TextureSubsystem still writes out ustrings, not ustringhashes!
+        // So we will need to go and convert that ustring to a hash
+        // for the rest of OSL to understand
+        auto ustr_data = reinterpret_cast<ustring*>(data);
+        (reinterpret_cast<ustringhash_pod*>(data))[0] = ustr_data->hash();
+    }
     if (!status) {
         std::string err = texturesys()->geterror();
         if (err.size()) {
@@ -525,4 +542,11 @@ RendererServices::batched(WidthOf<8>)
     return nullptr;
 }
 
-OSL_NAMESPACE_EXIT
+BatchedRendererServices<4>*
+RendererServices::batched(WidthOf<4>)
+{
+    // No default implementation for batched services
+    return nullptr;
+}
+
+OSL_NAMESPACE_END
