@@ -4,6 +4,10 @@
 # SPDX-License-Identifier: BSD-3-Clause
 # https://github.com/AcademySoftwareFoundation/OpenShadingLanguage
 
+failthresh = 0.01
+failpercent = 0.5
+hardfail = 0.035
+
 outputs = [ "out.exr" ]
 command = testrender("-r 256 256 -aa 4 --llvm_opt 13 bumptest.xml out.exr")
 
